# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties class for PH-HVAC DOAS Systems"""

try:
    from typing import Any
except:
    pass  # IronPython


class DOASSystemPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type)
        super(DOASSystemPhProperties_FromDictError, self).__init__(self.msg)


class DOASSystemPhProperties(object):
    """Honeybee-PH Properties for storing PH-style data."""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0

    @property
    def host(self):
        return self._host

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}
        if abridged:
            d['type'] = 'DOASSystemPhPropertiesAbridged'
        else:
            d['type'] = 'DOASSystemPhProperties'

        d['id_num'] = self.id_num
        return {'ph': d}

    @classmethod
    def from_dict(cls, _input_dict, host):
        # type: (dict, Any) -> DOASSystemPhProperties
        valid_types = ('DOASSystemPhProperties',
                       'DOASSystemPhPropertiesAbridged')
        if _input_dict['type'] not in valid_types:
            raise DOASSystemPhProperties_FromDictError(
                valid_types, _input_dict['type'])

        new_prop = cls(host)
        new_prop.id_num = _input_dict['id_num']
        return new_prop

    def apply_properties_from_dict(self, abridged_data):

        return

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Properties representation."""
        return '{}'.format(self.__class__.__name__)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""HBPH Heating Objects"""

import sys

try:
    from typing import Any, Optional, Sequence
except ImportError:
    pass  # IronPython 2.7

from honeybee_energy_ph.hvac import _base, fuels


class UnknownPhHeatingTypeError(Exception):
    def __init__(self, _heater_types, _received_type):
        # type: (list[str], str) -> None
        self.msg = 'Error: Unknown HBPH-Heating-SubSystem type? Got: "{}" but only types: {} are allowed?'.format(
            _received_type, _heater_types
        )
        super(UnknownPhHeatingTypeError, self).__init__(self.msg)


class PhHeatingSystem(_base._PhHVACBase):
    """Base class for all PH-Heating Systems (elec, boiler, etc...)"""

    def __init__(self):
        super(PhHeatingSystem, self).__init__()
        self.heating_type = self.__class__.__name__
        self.percent_coverage = 1.0

    def to_dict(self):
        # type: () -> dict
        d = {}
        d["identifier"] = self.identifier
        d["display_name"] = self.display_name
        d["heating_type"] = self.heating_type
        d["percent_coverage"] = self.percent_coverage
        return d

    def base_attrs_from_dict(self, _input_dict):
        # type: (PhHeatingSystem, dict) -> PhHeatingSystem
        self.identifier = _input_dict["identifier"]
        self.display_name = _input_dict["display_name"]
        self.heating_type = _input_dict["heating_type"]
        self.percent_coverage = _input_dict["percent_coverage"]
        return self

    def check_dict_type(self, _input_dict):
        # type: (dict) -> None
        """Check that the input dict type is correct for the Heating System being constructed."""
        heating_type = _input_dict["heating_type"]
        msg = (
            "Error creating Heating System from dict. Expected '{}' but got '{}'".format(
                self.__class__.__name__, heating_type
            )
        )
        assert heating_type == str(self.__class__.__name__), msg
        return None

    @classmethod
    def from_dict(cls, _input_dict):
        raise NotImplementedError("Error: from_dict() called on BaseClass.")

    def __lt__(self, other):
        # type: (PhHeatingSystem) -> bool
        return self.identifier < other.identifier


# -----------------------------------------------------------------------------
# Heating Types


class PhHeatingDirectElectric(PhHeatingSystem):
    """Heating via direct-electric (resistance heating)."""

    def __init__(self):
        super(PhHeatingDirectElectric, self).__init__()

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingDirectElectric, self).to_dict()
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingDirectElectric
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        return new_obj


class PhHeatingFossilBoiler(PhHeatingSystem):
    """Heating via boiler using fossil-fuel (gas, oil)"""

    def __init__(self):
        super(PhHeatingFossilBoiler, self).__init__()
        self.fuel = fuels.NATURAL_GAS
        self.condensing = True
        self.in_conditioned_space = True
        self.effic_at_30_percent_load = 0.98
        self.effic_at_nominal_load = 0.94
        self.avg_rtrn_temp_at_30_percent_load = 30
        self.avg_temp_at_70C_55C = 41
        self.avg_temp_at_55C_45C = 35
        self.avg_temp_at_32C_28C = 24

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingFossilBoiler, self).to_dict()

        d["fuel"] = self.fuel
        d["condensing"] = self.condensing
        d["in_conditioned_space"] = self.in_conditioned_space
        d["effic_at_30_percent_load"] = self.effic_at_30_percent_load
        d["effic_at_nominal_load"] = self.effic_at_nominal_load
        d["avg_rtrn_temp_at_30_percent_load"] = self.avg_rtrn_temp_at_30_percent_load
        d["avg_temp_at_70C_55C"] = self.avg_temp_at_70C_55C
        d["avg_temp_at_55C_45C"] = self.avg_temp_at_55C_45C
        d["avg_temp_at_32C_28C"] = self.avg_temp_at_32C_28C

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingFossilBoiler
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict["fuel"]
        new_obj.condensing = _input_dict["condensing"]
        new_obj.in_conditioned_space = _input_dict["in_conditioned_space"]
        new_obj.effic_at_30_percent_load = _input_dict["effic_at_30_percent_load"]
        new_obj.effic_at_nominal_load = _input_dict["effic_at_nominal_load"]
        new_obj.avg_rtrn_temp_at_30_percent_load = _input_dict[
            "avg_rtrn_temp_at_30_percent_load"
        ]
        new_obj.avg_temp_at_70C_55C = _input_dict["avg_temp_at_70C_55C"]
        new_obj.avg_temp_at_55C_45C = _input_dict["avg_temp_at_55C_45C"]
        new_obj.avg_temp_at_32C_28C = _input_dict["avg_temp_at_32C_28C"]

        return new_obj


class PhHeatingWoodBoiler(PhHeatingSystem):
    """Heating via boiler using wood (log, pellet)."""

    def __init__(self):
        super(PhHeatingWoodBoiler, self).__init__()
        self.fuel = fuels.WOOD_LOG
        self.in_conditioned_space = True
        self.effic_in_basic_cycle = 0.6
        self.effic_in_const_operation = 0.7
        self.avg_frac_heat_output = 0.4
        self.temp_diff_on_off = 30
        self.rated_capacity = 15  # kW
        self.demand_basic_cycle = 1  # kWh
        self.power_stationary_run = 1  # W
        self.power_standard_run = None  # type: Optional[float]
        self.no_transport_pellets = None  # type: Optional[bool]
        self.only_control = None  # type: Optional[bool]
        self.area_mech_room = None  # type: Optional[float]

    @property
    def useful_heat_output(self):
        return 0.9 * self.rated_capacity  # kWH

    @property
    def avg_power_output(self):
        return 0.5 * self.rated_capacity  # kW

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingWoodBoiler, self).to_dict()

        d["fuel"] = self.fuel
        d["in_conditioned_space"] = self.in_conditioned_space
        d["effic_in_basic_cycle"] = self.effic_in_basic_cycle
        d["effic_in_const_operation"] = self.effic_in_const_operation
        d["avg_frac_heat_output"] = self.avg_frac_heat_output
        d["temp_diff_on_off"] = self.temp_diff_on_off
        d["rated_capacity"] = self.rated_capacity
        d["demand_basic_cycle"] = self.demand_basic_cycle
        d["power_stationary_run"] = self.power_stationary_run
        d["power_standard_run"] = self.power_standard_run
        d["no_transport_pellets"] = self.no_transport_pellets
        d["only_control"] = self.only_control
        d["area_mech_room"] = self.area_mech_room

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingWoodBoiler

        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict["fuel"]
        new_obj.in_conditioned_space = _input_dict["in_conditioned_space"]
        new_obj.effic_in_basic_cycle = _input_dict["effic_in_basic_cycle"]
        new_obj.effic_in_const_operation = _input_dict["effic_in_const_operation"]
        new_obj.avg_frac_heat_output = _input_dict["avg_frac_heat_output"]
        new_obj.temp_diff_on_off = _input_dict["temp_diff_on_off"]
        new_obj.rated_capacity = _input_dict["rated_capacity"]
        new_obj.demand_basic_cycle = _input_dict["demand_basic_cycle"]
        new_obj.power_stationary_run = _input_dict["power_stationary_run"]
        new_obj.no_transport_pellets = _input_dict["no_transport_pellets"]
        new_obj.only_control = _input_dict["only_control"]
        new_obj.area_mech_room = _input_dict["area_mech_room"]

        return new_obj


class PhHeatingDistrict(PhHeatingSystem):
    """Heating via district-heat."""

    def __init__(self):
        super(PhHeatingDistrict, self).__init__()
        self.fuel = fuels.GAS_CGS_70_PHC
        self.util_factor_of_heat_transfer_station = 0.5

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingDistrict, self).to_dict()
        d["fuel"] = self.fuel
        d[
            "util_factor_of_heat_transfer_station"
        ] = self.util_factor_of_heat_transfer_station
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingDistrict
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.fuel = _input_dict["fuel"]
        new_obj.util_factor_of_heat_transfer_station = _input_dict[
            "util_factor_of_heat_transfer_station"
        ]
        return new_obj


class PhHeatingHeatPumpAnnual(PhHeatingSystem):
    def __init__(self):
        super(PhHeatingHeatPumpAnnual, self).__init__()
        self.annual_COP = 2.5
        self.total_system_perf_ratio = 0.4

    def to_dict(self):
        # type: () -> dict
        d = super(PhHeatingHeatPumpAnnual, self).to_dict()
        d["annual_COP"] = self.annual_COP
        d["total_system_perf_ratio"] = self.total_system_perf_ratio
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHeatingHeatPumpAnnual
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.annual_COP = _input_dict["annual_COP"]
        new_obj.total_system_perf_ratio = _input_dict["total_system_perf_ratio"]
        return new_obj


class PhHeatingHeatPumpRatedMonthly(PhHeatingSystem):
    """Heating via electric heat-pump."""

    def __init__(self):
        super(PhHeatingHeatPumpRatedMonthly, self).__init__()
        self.COP_1 = 2.5
        self.ambient_temp_1 = -8.333  # =17F
        self.COP_2 = 2.5
        self.ambient_temp_2 = 8.333  # =47F

    @property
    def monthly_COPS(self):
        return [self.COP_1, self.COP_2]

    @monthly_COPS.setter
    def monthly_COPS(self, _cops):
        # type: (Sequence[float]) -> None
        self.COP_1 = _cops[0]
        try:
            self.COP_2 = _cops[1]
        except IndexError:
            self.COP_2 = _cops[0]
        return

    @property
    def monthly_temps(self):
        return [self.ambient_temp_1, self.ambient_temp_2]

    @monthly_temps.setter
    def monthly_temps(self, _cops):
        # type: (Sequence[float]) -> None
        self.ambient_temp_1 = _cops[0]
        try:
            self.ambient_temp_2 = _cops[1]
        except IndexError:
            self.ambient_temp_2 = _cops[0]
        return

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhHeatingHeatPumpRatedMonthly, self).to_dict()
        d["COP_1"] = self.COP_1
        d["ambient_temp_1"] = self.ambient_temp_1
        d["COP_2"] = self.COP_2
        d["ambient_temp_2"] = self.ambient_temp_2
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingHeatPumpRatedMonthly
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)

        new_obj.COP_1 = _input_dict["COP_1"]
        new_obj.ambient_temp_1 = _input_dict["ambient_temp_1"]
        new_obj.COP_2 = _input_dict["COP_2"]
        new_obj.ambient_temp_2 = _input_dict["ambient_temp_2"]
        return new_obj


class PhHeatingHeatPumpCombined(PhHeatingSystem):
    def __init__(self):
        raise NotImplementedError()


# -----------------------------------------------------------------------------


class PhHeatingSystemBuilder(object):
    """Constructor class for PH-HeatingSystems"""

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhHeatingSystem
        """Find the right appliance constructor class from the module based on the 'type' name."""
        heating_type = _input_dict.get("heating_type")

        valid_class_types = [
            nm for nm in dir(sys.modules[__name__]) if nm.startswith("Ph")
        ]

        if heating_type not in valid_class_types:
            raise UnknownPhHeatingTypeError(valid_class_types, heating_type)
        heating_class = getattr(sys.modules[__name__], heating_type)
        new_equipment = heating_class.from_dict(_input_dict)
        return new_equipment

    def __str__(self):
        return "{}()".format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- Python Version: 2.7 -*-
# -*- coding: utf-8 -*-

"""HBPH Window Objects"""

try:
    from typing import Any, List
except ImportError:
    pass  # Python 2.7

from honeybee_energy_ph.construction import _base


class PhWindowFrameElement(_base._Base):
    """An individual PhWindowFrameElement (side) which is part of a complete PhWindowFrame."""

    def __init__(self, _identifier):
        super(PhWindowFrameElement, self).__init__(_identifier)
        self.width = 0.1
        self.u_factor = 1.0
        self.psi_glazing = 0.04
        self.psi_install = 0.04
        self.chi_value = 0.0

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhWindowFrameElement, self).to_dict()
        d['width'] = self.width
        d['u_factor'] = self.u_factor
        d['psi_glazing'] = self.psi_glazing
        d['psi_install'] = self.psi_install
        d['chi_value'] = self.chi_value
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhWindowFrameElement
        new_obj = cls(_input_dict['identifier'])
        new_obj.set_base_attrs_from_dict(_input_dict)
        new_obj.width = _input_dict['width']
        new_obj.u_factor = _input_dict['u_factor']
        new_obj.psi_glazing = _input_dict['psi_glazing']
        new_obj.psi_install = _input_dict['psi_install']
        new_obj.chi_value = _input_dict['chi_value']
        return new_obj

    def duplicate(self):
        # type: () -> PhWindowFrameElement
        return self.__copy__()

    def __copy__(self):
        # type: () -> PhWindowFrameElement
        new_obj = self.__class__(self.identifier)
        new_obj.set_base_attrs_from_obj(self)
        new_obj.width = self.width
        new_obj.u_factor = self.u_factor
        new_obj.psi_glazing = self.psi_glazing
        new_obj.psi_install = self.psi_install
        new_obj.chi_value = self.chi_value
        return new_obj

    def __str__(self):
        return '{}(width={:.3f}, u_factor={:.3f}, psi_glazing={:.3f}, psi_install={:.3f}, chi_value={:.3f})'.format(
            self.__class__.__name__, self.width, self.u_factor, self.psi_glazing, self.psi_install, self.chi_value)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class PhWindowFrame(_base._Base):
    """A full PH window frame made of 4 PhWindowFrameElements."""

    def __init__(self, _identifier):
        super(PhWindowFrame, self).__init__(_identifier)
        self.top = PhWindowFrameElement('PhWindowFrameElement')
        self.right = PhWindowFrameElement('PhWindowFrameElement')
        self.bottom = PhWindowFrameElement('PhWindowFrameElement')
        self.left = PhWindowFrameElement('PhWindowFrameElement')

    @property
    def elements(self):
        # type: () -> List[PhWindowFrameElement]
        """Return all four frame elements in clockwise order from top (t, r, b, l)."""
        return [self.top, self.right, self.bottom, self.left]

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhWindowFrame, self).to_dict()
        d['top'] = self.top.to_dict()
        d['right'] = self.right.to_dict()
        d['bottom'] = self.bottom.to_dict()
        d['left'] = self.left.to_dict()
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhWindowFrame
        new_obj = cls(_input_dict['identifier'])
        new_obj.set_base_attrs_from_dict(_input_dict)
        new_obj.top = PhWindowFrameElement.from_dict(_input_dict['top'])
        new_obj.right = PhWindowFrameElement.from_dict(_input_dict['right'])
        new_obj.bottom = PhWindowFrameElement.from_dict(_input_dict['bottom'])
        new_obj.left = PhWindowFrameElement.from_dict(_input_dict['left'])
        return new_obj

    def duplicate(self):
        # type: () -> PhWindowFrame
        return self.__copy__()

    def __copy__(self):
        # type: () -> PhWindowFrame
        new_obj = self.__class__(self.identifier)
        new_obj.set_base_attrs_from_obj(self)
        new_obj.top = self.top.duplicate()
        new_obj.right = self.right.duplicate()
        new_obj.bottom = self.bottom.duplicate()
        new_obj.left = self.left.duplicate()
        return new_obj

    def __str__(self):
        return '{}(top={!r}, right={!r}, bottom={!r}, left={!r})'.format(
            self.__class__.__name__, self.top, self.right, self.bottom, self.left)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class PhWindowGlazing(_base._Base):
    def __init__(self, _identifier):
        super(PhWindowGlazing, self).__init__(_identifier)
        self.u_factor = 1.0
        self.g_value = 0.4

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhWindowGlazing, self).to_dict()
        d['u_factor'] = self.u_factor
        d['g_value'] = self.g_value
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhWindowGlazing
        new_obj = cls(_input_dict['identifier'])
        new_obj.set_base_attrs_from_dict(_input_dict)
        new_obj.u_factor = _input_dict['u_factor']
        new_obj.g_value = _input_dict['g_value']
        return new_obj

    def duplicate(self):
        # type: () -> PhWindowGlazing
        return self.__copy__()

    def __copy__(self):
        # type: () -> PhWindowGlazing
        new_obj = self.__class__(self.identifier)
        new_obj.set_base_attrs_from_obj(self)
        new_obj.u_factor = self.u_factor
        new_obj.g_value = self.g_value
        return new_obj

    def __str__(self):
        return '{}(u_factor={:.3f}, g_value={:.3f})'.format(
            self.__class__.__name__, self.u_factor, self.g_value)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties for Honeybee-Energy-PH | Load | People"""

from uuid import uuid4

try:
    from typing import Any, Optional, Dict
except ImportError:
    pass  # Python 2.7


class PeoplePhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type
        )
        super(PeoplePhProperties_FromDictError, self).__init__(self.msg)


class PhDwellings(object):
    """A Dwelling Object to store information on the number of dwelling units."""

    _default = None # type: Optional[PhDwellings]

    def __init__(self, _num_dwellings=0):
        # type: (int) -> None
        self.identifier = uuid4()
        self._num_dwellings = _num_dwellings

    @property
    def num_dwellings(self):
        # type: () -> int
        return self._num_dwellings

    def to_dict(self, abridged=False):
        # type: (bool) -> Dict[str, Any]
        d = {}
        d["identifier"] = str(self.identifier)
        d["num_dwellings"] = self.num_dwellings
        return d
    
    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhDwellings
        obj = cls(_input_dict["num_dwellings"])
        obj.identifier = _input_dict["identifier"]
        return obj

    @classmethod
    def default(cls):
        # type: () -> PhDwellings
        if cls._default is None:
            cls._default = cls()
        return cls._default

    def duplicate(self, new_host=None):
        # type: (Any) -> PhDwellings
        obj = self.__class__(self.num_dwellings)
        obj.identifier = self.identifier
        return obj

    def __hash__(self):
        return hash(self.identifier)
    
    def __eq__(self, other):
        # type: (PhDwellings) -> bool
        return self.identifier == other.identifier

    def __str__(self):
        return '{}(num_dwellings={})'.format(self.__class__.__name__, self.num_dwellings)
    
    def __repr__(self):
        return str(self)
    
    def ToString(self):
        return str(self)


class PeoplePhProperties(object):
    """Ph Properties Object for Honeybee-Energy People"""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0
        self.number_bedrooms = 0
        self.number_people = 0.0
        self.dwellings = PhDwellings.default()

    def duplicate(self, new_host=None):
        # type: (Any) -> PeoplePhProperties
        _host = new_host or self._host
        new_properties_obj = self.__class__(_host)
        new_properties_obj.id_num = self.id_num
        new_properties_obj.number_bedrooms = self.number_bedrooms
        new_properties_obj.number_people = self.number_people
        new_properties_obj.dwellings = self.dwellings

        return new_properties_obj

    @property
    def host(self):
        return self._host

    @property
    def is_residential(self):
        # type: () -> bool
        """Return True is this Load is for a 'Residential' zone."""
        return self.dwellings.num_dwellings >= 1

    @property
    def is_dwelling_unit(self):
        print("WARNING: The 'PeoplePhProperties' property 'is_dwelling_unit' is deprecated and should be replace with 'is_residential' from now on.")
        return self.is_residential

    @property
    def number_dwelling_units(self):
        """Return the total number of dwelling units on the Load."""
        # type: () -> int
        return self.dwellings.num_dwellings

    def to_dict(self, abridged=False):
        # type: (bool) -> Dict[str, dict]
        d = {}

        if abridged:
            d["type"] = "PeoplePhPropertiesAbridged"
        else:
            d["type"] = "PeoplePhProperties"

        d["id_num"] = self.id_num
        d["number_bedrooms"] = self.number_bedrooms
        d["number_people"] = self.number_people
        d["dwellings"] = self.dwellings.to_dict()

        return {"ph": d}

    @classmethod
    def from_dict(cls, data, host):
        # type: (Dict[str, Any], Any) -> PeoplePhProperties
        valid_types = ("PeoplePhProperties", "PeoplePhPropertiesAbridged")
        if data["type"] not in valid_types:
            raise PeoplePhProperties_FromDictError(valid_types, data["type"])

        new_prop = cls(host)
        new_prop.id_num = data["id_num"]
        new_prop.number_bedrooms = data["number_bedrooms"]
        new_prop.number_people = data["number_people"]
        new_prop.dwellings = PhDwellings.from_dict(data["dwellings"])

        return new_prop

    def __eq__(self, other):
        # type: (PeoplePhProperties) -> bool

        if self.host != other.host:
            return False
        if self.id_num != other.id_num:
            return False
        if self.number_bedrooms != other.number_bedrooms:
            return False
        if self.number_people != other.number_people:
            return False
        if self.dwellings != other.dwellings:
            return False
        return True
    
    def __str__(self):
        return "{}: id={}".format(self.__class__.__name__, self.id_num)

    def __repr__(self):
        return (
            "{!r}(id_num={!r}, number_bedrooms={!r}"
            " number_people={!r}, dwellings={!r})".format(
                self.__class__.__name__,
                self.id_num,
                self.number_bedrooms,
                self.number_people,
                self.dwellings
            )
        )

    def ToString(self):
        return self.__repr__()



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Find Phius program data and build HBE-Programs."""

try:
    from honeybee.typing import clean_ep_string
except ImportError as e:
    raise ImportError("\nFailed to import honeybee:\n\t{}".format(e))

try:
    from honeybee_energy.load import lighting, people, equipment
    from honeybee_energy.schedule.ruleset import ScheduleRuleset
    from honeybee_energy.programtype import ProgramType
    from honeybee_energy.lib.scheduletypelimits import schedule_type_limit_by_identifier
    from honeybee_energy.lib.programtypes import (
        program_type_by_identifier,
        building_program_type_by_identifier,
    )
except ImportError as e:
    raise ImportError("\nFailed to import honeybee_energy:\n\t{}".format(e))

try:
    from honeybee_ph_standards.programtypes import PHIUS_programs
except ImportError as e:
    raise ImportError("\nFailed to import honeybee_energy_ph:\n\t{}".format(e))


try:
    from honeybee_energy_ph.properties import ruleset
    from honeybee_energy_ph.properties.load.lighting import LightingPhProperties
except ImportError as e:
    raise ImportError("\nFailed to import honeybee_energy_ph:\n\t{}".format(e))


class MissingBaseProgramError(Exception):
    def __init__(self, _data):
        self.msg = (
            'Error: The Phius data set for "{}" appears to missing '
            "a Honeybee base-program?".format(_data["name"])
        )
        super(MissingBaseProgramError, self).__init__(self.msg)


def _clean_str(_str):
    # type: (str) -> str
    if not _str:
        return ""
    else:
        return str(_str).lstrip().rstrip().upper().replace(" ", "_").replace("-", "_")


def load_data_from_Phius_standards(_search_key, _search_field="name", _protocol=""):
    # type: (str, str, str) -> list[dict[str, dict]]
    """Returns a list of Phius program data as dicts.

    Arguments:
    ----------
        * _search_key (str): The key to search the Phius library for.
        * _search_field (str): default='name' | The dict field to search.
        * _protocol (str): Optional extra flag to filter by the specific 'protocol'

    Returns:
    --------
        * (list[dict]): A list of the Phius data found in the library.
    """
    if not _search_key:
        return []

    prog_data = []
    _search_key = _clean_str(_search_key)
    _protocol = _clean_str(_protocol)
    for data in PHIUS_programs.PHIUS_library.values():
        if _search_key not in _clean_str(data[_search_field]):
            continue
        if not _protocol:
            prog_data.append(data)
        else:
            if _protocol in _clean_str(data["protocol"]):
                prog_data.append(data)

    return prog_data


def build_hb_people_from_Phius_data(_data):
    # type: (dict) -> people.People
    """Returns a new HBE-People Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "people" element from a full Phius data dict.

    Returns:
    --------
        * (people.People): A new HB-People object.
    """
    # -------------------------------------------------------------------------
    # -- Build the HBEPH Occupancy Schedule Properties
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties.from_days_per_year(
        _days_per_year=_data["schedule"]["annual_utilization_days"], _host=None
    )
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_start_end_hours(
            _start_hr=_data["schedule"]["start_hour"],
            _end_hr=_data["schedule"]["end_hour"],
            _op_frac=_data["schedule"]["relative_utilization_factor"],
            _name="_unnamed_op_period_",
        )
    )

    # -------------------------------------------------------------------------
    # -- Build the HBE Occupancy Schedule
    name = "{}::Occupancy".format(_data["schedule"]["name"])
    hbe_occupancy_schedule = ScheduleRuleset.from_constant_value(
        identifier=name,
        value=ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit=schedule_type_limit_by_identifier("Fractional"),
    )
    ph_sched_ph_prop._host = hbe_occupancy_schedule._properties
    hbe_occupancy_schedule._properties._ph = ph_sched_ph_prop  # type: ignore

    # -------------------------------------------------------------------------
    # -- Build the HBE-People Program
    hb_people = people.People(
        identifier=_data["loads"]["identifier"],
        people_per_area=_data["loads"]["people_per_area"],
        occupancy_schedule=hbe_occupancy_schedule,
    )
    return hb_people


def build_hb_lighting_from_Phius_data(_data):
    # type: (dict) -> lighting.Lighting
    """Returns a new HBE-Lighting Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "lighting" element from a full Phius data dict.

    Returns:
    --------
        * (lighting.Lighting): A new HB-Lighting object.
    """
    # -------------------------------------------------------------------------
    # -- Build the HBPH Lighting Schedule Properties
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties.from_days_per_year(
        _data["schedule"]["annual_utilization_days"], _host=None
    )
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_operating_hours(
            _data["schedule"]["daily_operating_hours"],
            _data["schedule"]["relative_utilization_factor"],
            "operating",
        )
    )

    # -------------------------------------------------------------------------
    # -- Build the HBE-Lighting Schedule
    name = "{}::Lighting".format(_data["schedule"]["name"])
    lighting_sched = ScheduleRuleset.from_constant_value(
        identifier=name,
        value=ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit=schedule_type_limit_by_identifier("Fractional"),
    )
    ph_sched_ph_prop._host = lighting_sched._properties
    lighting_sched._properties._ph = ph_sched_ph_prop  # type: ignore

    # -------------------------------------------------------------------------
    # -- Build the HBE-Lighting and set attributes
    # -- Phius standard datasets are all in IP units (W/ft2)
    # -- if any data is found with W/ft2 inputs, convert them
    try:
        # Convert W/ft2 to W/M2
        w_m2 = _data["loads"]["watts_per_ft2"] * 0.092903
    except KeyError:
        w_m2 = _data["loads"]["watts_per_m2"]

    hb_lighting = lighting.Lighting(
        _data["loads"]["name"], w_m2, lighting_sched, 0.0, 0.32, 0.25
    )

    # -- Set LightingPhProperties attributes
    hb_light_prop_ph = (
        hb_lighting.properties.ph # type: ignore
    )  # type: LightingPhProperties 
    hb_light_prop_ph.target_lux = _data["loads"]["target_lux"]
    hb_light_prop_ph.target_lux_height = _data["loads"]["target_lux_height"]

    return hb_lighting


def build_hb_elec_equip_from_Phius_data(_data):
    # type: (dict) -> equipment.ElectricEquipment
    """Returns a new HBE-Equipment Object with attributes based on the input data dict

    Arguments:
    ----------
        * _data (dict): The "elec_equipment" element from a full Phius data dict.

    Returns:
    --------
        * (people.People): A new HB-People object.
    """
    # -------------------------------------------------------------------------
    # -- Build the HBPH Elec Equip Schedule Properties
    ph_sched_ph_prop = ruleset.ScheduleRulesetPhProperties(_host=None)
    ph_sched_ph_prop.daily_operating_periods.add_period_to_collection(
        ruleset.DailyOperationPeriod.from_annual_utilization_factor(
            _data["schedule"]["annual_utilization_factor"], "operating"
        )
    )

    # -------------------------------------------------------------------------
    # -- Build the HBE-Electric Equip Schedule
    name = "{}::Elec_Equip".format(_data["schedule"]["name"])
    equip_sched = ScheduleRuleset.from_constant_value(
        identifier=name,
        value=ph_sched_ph_prop.annual_average_operating_fraction,
        schedule_type_limit=schedule_type_limit_by_identifier("Fractional"),
    )
    ph_sched_ph_prop._host = equip_sched._properties
    equip_sched._properties._ph = ph_sched_ph_prop  # type: ignore

    # -------------------------------------------------------------------------
    # -- Build the HBE-Electric Equip and set attributes

    # -- Phius standard datasets are all in IP units (W/ft2)
    # -- if any data is found with W/ft2 inputs, convert them
    try:
        # Convert W/ft2 to W/M2
        w_m2 = _data["loads"]["watts_per_ft2"] * 0.092903
    except KeyError:
        w_m2 = _data["loads"]["watts_per_m2"]

    hb_elec_equip = equipment.ElectricEquipment(
        _data["loads"]["identifier"], w_m2, equip_sched
    )

    return hb_elec_equip


def build_hb_program_from_Phius_data(_data):
    # type: (dict) -> ProgramType
    """Return a new HB-Program with attributes based on an input Phius dataset

    Arguments:
    ---------
        * _data (dict): The full Phius datadict

    Returns:
    --------
        * (ProgramType): The new Honeybee-Energy ProgramType.
    """

    # -- Sort out the base-program
    base_program = _data["hb_base_program"]

    if not base_program:
        raise MissingBaseProgramError(_data)
    try:
        base_program_ = building_program_type_by_identifier(base_program)
    except ValueError:
        base_program_ = program_type_by_identifier(base_program)

    program = base_program_.duplicate()
    program.identifier = clean_ep_string(
        "Phius_{}".format(_data["name"].replace(" ", "_"))
    )
    program.display_name = "{}::{}".format(_data["protocol"], _data["name"])

    # -- Build and assign the sub-programs
    program.people = build_hb_people_from_Phius_data(_data["people"])
    program.lighting = build_hb_lighting_from_Phius_data(_data["lighting"])
    program.electric_equipment = build_hb_elec_equip_from_Phius_data(
        _data["elec_equipment"]
    )

    # -- Inherit the rest from the base-program
    # program.gas_equipment = _gas_equip_
    # program.service_hot_water = _hot_water_
    # program.infiltration = _infiltration_
    # program.ventilation = _ventilation_
    # program.setpoint = _setpoint_

    return program



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Passive House HVAC Equipment Classes"""

import sys

try:
    from typing import Any, Optional, Union, Dict
except ImportError:
    pass  # IronPython

from honeybee_energy_ph.hvac import _base

# -----------------------------------------------------------------------------


class UnknownPhExhaustVentTypeError(Exception):
    def __init__(self, _device_types, _received_type):
        # type: (list[str], str) -> None
        self.msg = 'Error: Unknown HBPH-Exhaust Ventilation type? Got: "{}"\
            "but only types: {} are allowed?'.format(
            _received_type, _device_types
        )
        super(UnknownPhExhaustVentTypeError, self).__init__(self.msg)


# -----------------------------------------------------------------------------
# -- ERV units


class Ventilator(_base._PhHVACBase):
    def __init__(self):
        super(Ventilator, self).__init__()
        self.display_name = "_unnamed_ventilator_"
        self.id_num = 0
        self.quantity = 1
        self.sensible_heat_recovery = 0.0
        self.latent_heat_recovery = 0.0
        self.electric_efficiency = 0.55
        self.frost_protection_reqd = True
        self.temperature_below_defrost_used = -5
        self.in_conditioned_space = True

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d["display_name"] = self.display_name
        d["quantity"] = self.quantity
        d["sensible_heat_recovery"] = self.sensible_heat_recovery
        d["latent_heat_recovery"] = self.latent_heat_recovery
        d["electric_efficiency"] = self.electric_efficiency
        d["frost_protection_reqd"] = self.frost_protection_reqd
        d["temperature_below_defrost_used"] = self.temperature_below_defrost_used
        d["in_conditioned_space"] = self.in_conditioned_space

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> Ventilator
        obj = cls()

        obj.display_name = _input_dict["display_name"]
        obj.quantity = _input_dict["quantity"]
        obj.sensible_heat_recovery = _input_dict["sensible_heat_recovery"]
        obj.latent_heat_recovery = _input_dict["latent_heat_recovery"]
        obj.electric_efficiency = _input_dict["electric_efficiency"]
        obj.frost_protection_reqd = _input_dict["frost_protection_reqd"]
        obj.temperature_below_defrost_used = _input_dict["temperature_below_defrost_used"]
        obj.in_conditioned_space = _input_dict["in_conditioned_space"]

        return obj

    def __lt__(self, other):
        # type: (Ventilator) -> bool
        return self.identifier < other.identifier

    def __repr__(self):
        return "{}(display_name={!r}, sensible_heat_recovery={:0.2f})".format(
            self.__class__.__name__, self.display_name, self.sensible_heat_recovery
        )

    def ToString(self):
        return self.__repr__()


class PhVentilationSystem(_base._PhHVACBase):
    """Passive House Fresh-Air Ventilation System."""

    def __init__(self):
        super(PhVentilationSystem, self).__init__()
        self.display_name = "_unnamed_ph_vent_system_"
        self.sys_type = 1  # '1-Balanced PH ventilation with HR'
        self.duct_01 = None
        self.duct_02 = None
        self._ventilation_unit = None  # type: Optional[Ventilator]
        self.id_num = 0

    @property
    def ventilation_unit(self):
        # type: () -> Optional[Ventilator]
        return self._ventilation_unit

    @ventilation_unit.setter
    def ventilation_unit(self, _in):
        # type: (Optional[Ventilator]) -> None
        self._ventilation_unit = _in

        if not self._ventilation_unit:
            return None

        if self._ventilation_unit.display_name == "_unnamed_ventilator_":
            self._ventilation_unit.display_name = self.display_name
        return None

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d["identifier"] = str(self.identifier)
        d["name"] = self.display_name
        d["sys_type"] = self.sys_type
        d["duct_01"] = self.duct_01
        d["duct_02"] = self.duct_02
        d["id_num"] = self.id_num

        if self.ventilation_unit:
            d["ventilation_unit"] = self.ventilation_unit.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhVentilationSystem
        obj = cls()

        obj.identifier = _input_dict["identifier"]
        obj.display_name = _input_dict["name"]
        obj.sys_type = _input_dict["sys_type"]
        obj.duct_01 = _input_dict["duct_01"]
        obj.duct_02 = _input_dict["duct_02"]
        obj.id_num = _input_dict.get("id_num", 0)

        vent_unit_dict = _input_dict.get("ventilation_unit", None)
        if vent_unit_dict:
            obj.ventilation_unit = Ventilator.from_dict(vent_unit_dict)

        return obj

    def __lt__(self, other):
        # type: (PhVentilationSystem) -> bool
        return self.identifier < other.identifier

    def __repr__(self):
        return "{}(display_name={!r}, sys_type={!r})".format(
            self.__class__.__name__, self.display_name, self.sys_type
        )

    def ToString(self):
        return self.__repr__()


# -----------------------------------------------------------------------------
# -- Exhaust Ventilators are not part of the Ventilation System,
# -- but instead are treated more like appliances which get added to the Room.


class _ExhaustVentilatorBase(_base._PhHVACBase):
    def __init__(self):
        super(_ExhaustVentilatorBase, self).__init__()
        self.device_class_name = self.__class__.__name__
        self.display_name = "_unnamed_exhaust_ventilator_"
        self.quantity = 1
        self.annual_runtime_minutes = 0.0
        self.exhaust_flow_rate_m3s = 0.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d["device_class_name"] = self.device_class_name
        d["display_name"] = self.display_name
        d["identifier"] = self.identifier
        d["quantity"] = self.quantity
        d["annual_runtime_minutes"] = self.annual_runtime_minutes
        d["exhaust_flow_rate_m3s"] = self.exhaust_flow_rate_m3s

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> _ExhaustVentilatorBase
        new_obj = cls()

        new_obj.device_class_name = _input_dict["device_class_name"]
        new_obj.display_name = _input_dict["display_name"]
        new_obj.quantity = _input_dict["quantity"]
        new_obj.identifier = _input_dict["identifier"]
        new_obj.annual_runtime_minutes = _input_dict["annual_runtime_minutes"]
        new_obj.exhaust_flow_rate_m3s = _input_dict["exhaust_flow_rate_m3s"]

        return new_obj

    def __lt__(self, other):
        # type: (_ExhaustVentilatorBase) -> bool
        return self.identifier < other.identifier

    def __str__(self):
        return "{}(display_name={!r}, exhaust_flow_rate_m3s={:0.02f})".format(
            self.__class__.__name__, self.display_name, self.exhaust_flow_rate_m3s
        )

    def __repr__(self):
        return str(self)

    def ToString(self):
        return self.__repr__()


class ExhaustVentDryer(_ExhaustVentilatorBase):
    def __init__(self):
        super(ExhaustVentDryer, self).__init__()
        self.display_name = "_unnamed_dryer_exh_"


class ExhaustVentKitchenHood(_ExhaustVentilatorBase):
    def __init__(self):
        super(ExhaustVentKitchenHood, self).__init__()
        self.display_name = "_unnamed_kitchen_hood_exh_"


class ExhaustVentUserDefined(_ExhaustVentilatorBase):
    def __init__(self):
        super(ExhaustVentUserDefined, self).__init__()
        self.display_name = "_unnamed_user_defined_exh_"


class PhExhaustDeviceBuilder(object):
    """Constructor class for HBPH-Exhaust Ventilation Devices"""

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> _ExhaustVentilatorBase
        """Find the right device constructor class from the module based on the 'type' name."""
        device_class_name = _input_dict["device_class_name"]  # type: str

        valid_class_types = [
            nm for nm in dir(sys.modules[__name__]) if nm.startswith("ExhaustVent")
        ]

        if device_class_name not in valid_class_types:
            raise UnknownPhExhaustVentTypeError(valid_class_types, device_class_name)
        device_class = getattr(
            sys.modules[__name__], device_class_name
        )  # type: _ExhaustVentilatorBase
        new_equipment = device_class.from_dict(_input_dict)
        return new_equipment

    def __str__(self):
        return "{}()".format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        """Overwrite .NET ToString."""
        return repr(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""HBPH Cooling Objects"""

try:
    from typing import Any, List, Dict
except ImportError:
    pass  # IronPython 2.7

import sys

from honeybee_energy_ph.hvac import _base

# -----------------------------------------------------------------------------
# Cooling Base


class UnknownPhCoolingTypeError(Exception):
    def __init__(self, _heater_types, _received_type):
        # type: (List[str], str) -> None
        self.msg = 'Error: Unknown HBPH-Cooling-SubSystem type? Got: "{}" but only types: {} are allowed?'.format(
            _received_type, _heater_types
        )
        super(UnknownPhCoolingTypeError, self).__init__(self.msg)


class PhCoolingSystem(_base._PhHVACBase):
    """Base class for all HBPH-Cooling-Systems."""

    def __init__(self):
        super(PhCoolingSystem, self).__init__()
        self.cooling_class_name = self.__class__.__name__
        self.percent_coverage = 1.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}
        d["identifier"] = self.identifier
        d["display_name"] = self.display_name
        d["cooling_class_name"] = self.cooling_class_name
        d["percent_coverage"] = self.percent_coverage
        return d

    def base_attrs_from_dict(self, _input_dict):
        # type: (PhCoolingSystem, Dict[str, Any]) -> PhCoolingSystem
        self.identifier = _input_dict["identifier"]
        self.display_name = _input_dict["display_name"]
        self.cooling_class_name = _input_dict["cooling_class_name"]
        self.percent_coverage = _input_dict["percent_coverage"]
        return self

    def check_dict_type(self, _input_dict):
        # type: (Dict[str, Any]) -> None
        """Check that the input dict type is correct for the Cooling System being constructed."""
        cooling_class_name = _input_dict["cooling_class_name"]
        msg = (
            "Error creating Cooling System from dict. Expected '{}' but got '{}'".format(
                self.__class__.__name__, cooling_class_name
            )
        )
        assert cooling_class_name == str(self.__class__.__name__), msg
        return None

    @classmethod
    def from_dict(cls, input_dict):
        raise NotImplementedError("Error: from_dict() called from BaseClass")

    def __lt__(self, other):
        # type: (PhCoolingSystem) -> bool
        return self.identifier < other.identifier


# -----------------------------------------------------------------------------
# Cooling Types


class PhCoolingVentilation(PhCoolingSystem):
    """Cooling via the Fresh-Air Ventilation System (ERV)."""

    def __init__(self):
        super(PhCoolingVentilation, self).__init__()
        self.single_speed = False
        self.min_coil_temp = 12.0
        self.capacity = 10.0
        self.annual_COP = 4.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = super(PhCoolingVentilation, self).to_dict()
        d["single_speed"] = self.single_speed
        d["min_coil_temp"] = self.min_coil_temp
        d["capacity"] = self.capacity
        d["annual_COP"] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhCoolingVentilation
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.single_speed = _input_dict["single_speed"]
        new_obj.min_coil_temp = _input_dict["min_coil_temp"]
        new_obj.capacity = _input_dict["capacity"]
        new_obj.annual_COP = _input_dict["annual_COP"]
        return new_obj


class PhCoolingRecirculation(PhCoolingSystem):
    """Cooling via a 'recirculation' system (typical AC)."""

    def __init__(self):
        super(PhCoolingRecirculation, self).__init__()
        self.single_speed = False
        self.min_coil_temp = 12.0
        self.flow_rate_m3_hr = 100.0
        self.flow_rate_variable = True
        self.capacity = 10.0
        self.annual_COP = 4.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = super(PhCoolingRecirculation, self).to_dict()
        d["single_speed"] = self.single_speed
        d["min_coil_temp"] = self.min_coil_temp
        d["flow_rate_m3_hr"] = self.flow_rate_m3_hr
        d["flow_rate_variable"] = self.flow_rate_variable
        d["capacity"] = self.capacity
        d["annual_COP"] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhCoolingRecirculation
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.single_speed = _input_dict["single_speed"]
        new_obj.min_coil_temp = _input_dict["min_coil_temp"]
        new_obj.flow_rate_m3_hr = _input_dict["flow_rate_m3_hr"]
        new_obj.flow_rate_variable = _input_dict["flow_rate_variable"]
        new_obj.capacity = _input_dict["capacity"]
        new_obj.annual_COP = _input_dict["annual_COP"]
        return new_obj


class PhCoolingDehumidification(PhCoolingSystem):
    """Cooling via dedicated dehumidification system."""

    def __init__(self):
        super(PhCoolingDehumidification, self).__init__()
        self.useful_heat_loss = False
        self.annual_COP = 4.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = super(PhCoolingDehumidification, self).to_dict()
        d["useful_heat_loss"] = self.useful_heat_loss
        d["annual_COP"] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhCoolingDehumidification
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.useful_heat_loss = _input_dict["useful_heat_loss"]
        new_obj.annual_COP = _input_dict["annual_COP"]
        return new_obj


class PhCoolingPanel(PhCoolingSystem):
    """Cooling via radiant panels."""

    def __init__(self):
        super(PhCoolingPanel, self).__init__()
        self.annual_COP = 4.0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = super(PhCoolingPanel, self).to_dict()
        d["annual_COP"] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhCoolingPanel
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.annual_COP = _input_dict["annual_COP"]
        return new_obj


# -----------------------------------------------------------------------------


class PhCoolingSystemBuilder(object):
    """Constructor class for HBPH-CoolingSystems"""

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> PhCoolingSystem
        """Find the right cooling constructor class from the module based on the 'type' name."""
        cooling_class_name = _input_dict["cooling_class_name"]  # type: str

        valid_class_types = [
            nm for nm in dir(sys.modules[__name__]) if nm.startswith("Ph")
        ]

        if cooling_class_name not in valid_class_types:
            raise UnknownPhCoolingTypeError(valid_class_types, cooling_class_name)
        cooling_class = getattr(sys.modules[__name__], cooling_class_name)
        new_equipment = cooling_class.from_dict(_input_dict)
        return new_equipment

    def __str__(self):
        return "{}()".format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""This is called during __init__ and extends the base honeybee class Properties with a new ._ph slot"""

# -- honeybee_energy properties objects
from honeybee_energy.schedule.ruleset import ScheduleRulesetProperties
from honeybee_ph.properties.space import SpaceProperties
from honeybee_energy.properties.extension import OpaqueConstructionProperties, \
    EnergyMaterialProperties, EnergyMaterialNoMassProperties, EnergyMaterialVegetationProperties, \
    WindowConstructionProperties, IdealAirSystemProperties, AllAirSystemProperties, \
    HeatCoolSystemProperties, DOASSystemProperties, \
    ServiceHotWaterProperties, SHWSystemProperties, ElectricEquipmentProperties,\
    PeopleProperties, LightingProperties


# -- honeybee_energy_ph properties objects
from honeybee_energy_ph.properties.ruleset import ScheduleRulesetPhProperties
from honeybee_energy_ph.properties.space import SpaceEnergyProperties
from honeybee_energy_ph.properties.construction.opaque import OpaqueConstructionPhProperties
from honeybee_energy_ph.properties.materials.opaque import EnergyMaterialPhProperties, \
    EnergyMaterialNoMassPhProperties, EnergyMaterialVegetationPhProperties
from honeybee_energy_ph.properties.construction.window import WindowConstructionPhProperties
from honeybee_energy_ph.properties.hot_water.hw_program import ServiceHotWaterPhProperties
from honeybee_energy_ph.properties.hot_water.hw_system import SHWSystemPhProperties
from honeybee_energy_ph.properties.load.equipment import ElectricEquipmentPhProperties
from honeybee_energy_ph.properties.load.people import PeoplePhProperties
from honeybee_energy_ph.properties.load.lighting import LightingPhProperties
from honeybee_energy_ph.properties.hvac.idealair import IdealAirSystemPhProperties
from honeybee_energy_ph.properties.hvac.allair import AllAirSystemPhProperties
from honeybee_energy_ph.properties.hvac.doas import DOASSystemPhProperties
from honeybee_energy_ph.properties.hvac.heatcool import HeatCoolSystemPhProperties

# Step 1)
# set a private ._ph attribute on each relevant HB-Energy Property class to None
setattr(SpaceProperties, '_energy', None)
setattr(ScheduleRulesetProperties, '_ph', None)
setattr(OpaqueConstructionProperties, '_ph', None)
setattr(EnergyMaterialProperties, '_ph', None)
setattr(EnergyMaterialNoMassProperties, '_ph', None)
setattr(EnergyMaterialVegetationProperties, '_ph', None)
setattr(WindowConstructionProperties, '_ph', None)
setattr(ServiceHotWaterProperties, '_ph', None)
setattr(SHWSystemProperties, '_ph', None)
setattr(ElectricEquipmentProperties, '_ph', None)
setattr(PeopleProperties, '_ph', None)
setattr(LightingProperties, '_ph', None)
setattr(IdealAirSystemProperties, '_ph', None)
setattr(AllAirSystemProperties, '_ph', None)
setattr(DOASSystemProperties, '_ph', None)
setattr(HeatCoolSystemProperties, '_ph', None)

# Step 2)
# create methods to define the public .property.<extension> @property instances on each obj.properties container


def space_energy_properties(self):
    if self._energy is None:
        self._energy = SpaceEnergyProperties(self.host)
    return self._energy


def schedule_ruleset_ph_properties(self):
    if self._ph is None:
        self._ph = ScheduleRulesetPhProperties(self.host)
    return self._ph


def opaque_construction_ph_properties(self):
    if self._ph is None:
        self._ph = OpaqueConstructionPhProperties()
    return self._ph


def energy_material_ph_properties(self):
    if self._ph is None:
        self._ph = EnergyMaterialPhProperties()
    return self._ph


def energy_material_no_mass_ph_properties(self):
    if self._ph is None:
        self._ph = EnergyMaterialNoMassPhProperties()
    return self._ph


def energy_material_vegetation_ph_properties(self):
    if self._ph is None:
        self._ph = EnergyMaterialVegetationPhProperties()
    return self._ph


def window_construction_ph_properties(self):
    if self._ph is None:
        self._ph = WindowConstructionPhProperties(self.host)
    return self._ph


def hot_water_program_ph_properties(self):
    if self._ph is None:
        self._ph = ServiceHotWaterPhProperties(self.host)
    return self._ph


def hot_water_system_ph_properties(self):
    if self._ph is None:
        self._ph = SHWSystemPhProperties(self.host)
    return self._ph


def elec_equip_ph_properties(self):
    if self._ph is None:
        self._ph = ElectricEquipmentPhProperties(self.host)
    return self._ph


def people_ph_properties(self):
    if self._ph is None:
        self._ph = PeoplePhProperties(self.host)
    return self._ph


def lighting_ph_properties(self):
    if self._ph is None:
        self._ph = LightingPhProperties(self.host)
    return self._ph


def hvac_ideal_air_system_ph_properties(self):
    if self._ph is None:
        self._ph = IdealAirSystemPhProperties(self.host)
    return self._ph


def hvac_all_air_system_ph_properties(self):
    if self._ph is None:
        self._ph = AllAirSystemPhProperties(self.host)
    return self._ph


def hvac_doas_system_ph_properties(self):
    if self._ph is None:
        self._ph = DOASSystemPhProperties(self.host)
    return self._ph


def hvac_heatcool_system_ph_properties(self):
    if self._ph is None:
        self._ph = HeatCoolSystemPhProperties(self.host)
    return self._ph


# Step 3)
# add public .energy or .ph @property methods to the appropriate Properties classes
setattr(SpaceProperties, 'energy', property(space_energy_properties))
setattr(ScheduleRulesetProperties, 'ph', property(schedule_ruleset_ph_properties))
setattr(OpaqueConstructionProperties, 'ph', property(opaque_construction_ph_properties))
setattr(WindowConstructionProperties, 'ph', property(window_construction_ph_properties))
setattr(EnergyMaterialProperties, 'ph', property(energy_material_ph_properties))
setattr(EnergyMaterialNoMassProperties, 'ph',
        property(energy_material_no_mass_ph_properties))
setattr(EnergyMaterialVegetationProperties, 'ph',
        property(energy_material_vegetation_ph_properties))
setattr(ServiceHotWaterProperties, 'ph', property(hot_water_program_ph_properties))
setattr(SHWSystemProperties, 'ph', property(hot_water_system_ph_properties))
setattr(ElectricEquipmentProperties, 'ph', property(elec_equip_ph_properties))
setattr(PeopleProperties, 'ph', property(people_ph_properties))
setattr(LightingProperties, 'ph', property(lighting_ph_properties))
setattr(IdealAirSystemProperties, 'ph', property(hvac_ideal_air_system_ph_properties))
setattr(AllAirSystemProperties, 'ph', property(hvac_all_air_system_ph_properties))
setattr(DOASSystemProperties, 'ph', property(hvac_doas_system_ph_properties))
setattr(HeatCoolSystemProperties, 'ph', property(hvac_heatcool_system_ph_properties))


# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Properties class for PH-HVAC IdealAir Systems"""

try:
    from typing import Any, Optional, Dict
except:
    pass  # IronPython

from honeybee_energy_ph.hvac import ventilation, heating, cooling


class IdealAirSystemPhProperties_FromDictError(Exception):
    def __init__(self, _expected_types, _input_type):
        self.msg = 'Error: Expected type of "{}". Got: {}'.format(
            _expected_types, _input_type
        )
        super(IdealAirSystemPhProperties_FromDictError, self).__init__(self.msg)


class IdealAirSystemPhProperties(object):
    """Honeybee-PH Properties for storing PH-style data."""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0
        self.ventilation_system = None  # type: Optional[ventilation.PhVentilationSystem]
        self.heating_systems = set()  # type: set[heating.PhHeatingSystem]
        self.cooling_systems = set()  # type: set[cooling.PhCoolingSystem]
        self.exhaust_vent_devices = set()  # type: set[ventilation._ExhaustVentilatorBase]

    @property
    def host(self):
        return self._host

    def to_dict(self, abridged=False):
        # type: (bool) -> dict[str, dict]
        d = {}
        if abridged:
            d["type"] = "IdealAirSystemPhPropertiesAbridged"
        else:
            d["type"] = "IdealAirSystemPhProperties"

        d["id_num"] = self.id_num

        if self.ventilation_system:
            d["ventilation_system"] = self.ventilation_system.to_dict()
        else:
            d["ventilation_system"] = None

        d["heating_systems"] = [
            sys.to_dict()
            for sys in sorted([_ for _ in self.heating_systems if _ is not None])
        ]

        d["cooling_systems"] = [
            sys.to_dict()
            for sys in sorted([_ for _ in self.cooling_systems if _ is not None])
        ]

        d["exhaust_vent_devices"] = [
            sys.to_dict()
            for sys in sorted([_ for _ in self.exhaust_vent_devices if _ is not None])
        ]

        return {"ph": d}

    @classmethod
    def from_dict(cls, _input_dict, host):
        # type: (Dict[str, Any], Any) -> IdealAirSystemPhProperties
        valid_types = ("IdealAirSystemPhProperties", "IdealAirSystemPhPropertiesAbridged")
        if _input_dict["type"] not in valid_types:
            raise IdealAirSystemPhProperties_FromDictError(
                valid_types, _input_dict["type"]
            )

        new_prop = cls(host)
        new_prop.id_num = _input_dict["id_num"]

        vent_sys_dict = _input_dict["ventilation_system"]
        if vent_sys_dict:
            ph_vent_sys = ventilation.PhVentilationSystem.from_dict(vent_sys_dict)
            new_prop.ventilation_system = ph_vent_sys

        for htg_sys_dict in _input_dict.get("heating_systems", []):
            htg_sys = heating.PhHeatingSystemBuilder.from_dict(htg_sys_dict)
            new_prop.heating_systems.add(htg_sys)

        for cooling_sys_dict in _input_dict.get("cooling_systems", []):
            cooling_sys = cooling.PhCoolingSystemBuilder.from_dict(cooling_sys_dict)
            new_prop.cooling_systems.add(cooling_sys)

        for exhaust_vent_device_dict in _input_dict.get("exhaust_vent_devices", []):
            exhaust_device = ventilation.PhExhaustDeviceBuilder.from_dict(
                exhaust_vent_device_dict
            )
            new_prop.exhaust_vent_devices.add(exhaust_device)

        return new_prop

    def apply_properties_from_dict(self, abridged_data):
        return

    def duplicate(self, new_host=None):
        # type: (Any) -> IdealAirSystemPhProperties
        return self.__copy__(new_host)

    def __copy__(self, new_host=None):
        # type: (Any) -> IdealAirSystemPhProperties
        _host = new_host or self._host
        new_obj = IdealAirSystemPhProperties(_host)

        new_obj.id_num = self.id_num
        new_obj.ventilation_system = self.ventilation_system

        for htg_sys in self.heating_systems:
            new_obj.heating_systems.add(htg_sys)

        for clg_sys in self.cooling_systems:
            new_obj.cooling_systems.add(clg_sys)

        for exhaust_device in self.exhaust_vent_devices:
            new_obj.exhaust_vent_devices.add(exhaust_device)

        return new_obj

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Properties representation."""
        return "{}".format(self.__class__.__name__)
